#! /usr/bin/env python3
import os
import sys

import typer
from capstone import *
from base64 import b64decode as d64

app = typer.Typer()
ARCHS = {k: v for k, v in locals().items() if k.startswith("CS_ARCH")}
MODES = {k: v for k, v in locals().items() if k.startswith("CS_MODE")}
ARCHS_TEXT = "\n\n\t- ".join(["", *ARCHS.keys()])
MODES_TEXT = "\n\n\t- ".join(["", *MODES.keys()])


def secho(address: int, mnemonic: int, op_str: int) -> None:
    """
    Print an disassembled instruction with colors.
    TODO: colors look probably shitty and need to be tweaked

    :param address:
    :param mnemonic:
    :param op_str:
    :return:
    """
    typer.echo("\t".join((
        typer.style('0x%x:' % address, fg='blue'),
        typer.style('%s' % mnemonic, fg='green'),
        typer.style('%s' % op_str, fg='yellow'),
    )))


def echo(address: int, mnemonic: int, op_str: int) -> None:
    """
    Print an disassembled instruction without colors.

    :param address:
    :param mnemonic:
    :param op_str:
    :return:
    """
    typer.echo("0x%x:\t%s\t%s" % (address, mnemonic, op_str))


@app.callback(
    invoke_without_command=True,
    help=f"""Supported archs are (default: CS_ARCH_X86):
{ARCHS_TEXT}

Supported modes are (default: CS_MODE_32):
{MODES_TEXT}
"""
)
def to_asm(
        arch: str = "CS_ARCH_X86",
        mode: str = "CS_MODE_32",
        color: bool = True,
        verbose: bool = False,

):
    log = echo if not color else secho

    try:
        arch_type = ARCHS[arch]
        if verbose:
            typer.secho(f"Selected arch type: {arch_type}", fg='green')

    except KeyError:
        typer.secho(f"Could not find arch {arch} in the following list:", fg='red')
        typer.secho(f"{ARCHS_TEXT}", fg='red')
        sys.exit(os.EX_SOFTWARE)

    try:
        mode_type = MODES[mode]
        if verbose:
            typer.secho(f"Selected mode type: {mode_type}", fg='green')

    except KeyError:
        typer.secho(f"Could not find mode {mode} in the following list:", fg='red')
        typer.secho(f"{MODES_TEXT}", fg='red')
        sys.exit(os.EX_SOFTWARE)

    dis = Cs(arch_type, mode_type)

    # TODO: deal with bad shellcode characters here
    shellcode = d64(sys.stdin.buffer.read())
    if verbose:
        typer.secho(f"Read shellcode: {shellcode}", fg='yellow')

    for instruction in dis.disasm(shellcode, 0x00):
        log(instruction.address, instruction.mnemonic, instruction.op_str)


if __name__ == "__main__":
    app()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['shellcode']

package_data = \
{'': ['*']}

install_requires = \
['capstone>=4.0.2,<5.0.0', 'typer>=0.3.2,<0.4.0']

setup_kwargs = {
    'name': 'shellcode',
    'version': '0.1.3',
    'description': 'CLI to turn shellcode back to asm.',
    'long_description': '# shellcode\n![shellcode logo](https://raw.githubusercontent.com/4thel00z/shellcode/master/logo.png)\n\n## Motivation\n\nIn the course of pawning n00bs often the need arises to introspect some (generated) shellcode or check\nit for sanity.\n\nThis tools does exactly that.\n\n## Usage\n```\necho "<some shellcode>" | python3 -m shellcode \n```\n\n## License\n\nThis project is licensed under the GPL-3 license.\n',
    'author': '4thel00z',
    'author_email': '4thel00z@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/4thel00z/shellcode',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
